#ifndef SPARSE_ARRAY_HPP
#define SPARSE_ARRAY_HPP
/**
 * @class SparseArray
 * @brief Structures and Headers of methods for a working with SparseArrays
 * @author Max Hadersbeck, based on Ideas from Florain Fink
 *
 * defines Structures and Header of methods, which are necessary to
 * work with sparse Arrays
 *
 */


#include <stdexcept>
#include "sparse_cell.hpp"

/**
 * @brief Dynamic growing array of Cells.
 *
 * Internally the Cells structure uses
 * the c routines calloc(), memset(), realloc() and free().
 * New Cells are all set to 0.
 * @author Florian Fink
 */
class SparseArray {
public:
	/**
	 * Constructor.
	 */
	SparseArray();

	/**
	 * Constructor.
	 * Constructs a DynArray with a initial size of n.
	 * All cells in the array are initialized to 0.
	 * @param[IN] n The initial size of the internal data array.
	 */
	SparseArray(std::size_t n);

	/**
	 * Copy Constructor.
	 * Copies all internal data.
	 * @param[IN] rhs A constant reference to the other DynArray object.
	 */
	SparseArray(const SparseArray& rhs);

	/**
	 * Overloaded '=' operator.
	 * Copies all internal data.
	 * @param[IN] rhs A constant reference to the other DynArray object.
	 */
	SparseArray& operator=(const SparseArray& rhs);

	/**
	 * Destructor.
	 */
	~SparseArray();

	/**
	 * Get a reference to the cell at the given index.
	 * This method does *not* check if the index is
	 * in the range of the array.
	 * @param[IN] index The index in the array.
	 * @return A reference to the cell.
	 */
	SparseCell& operator[](std::size_t index);

	/**
	 * Get a reference to the cell at the given index.
	 * This method checks if the index is in the range
	 * of the array and re-sizes the array accordingly.
	 * @param[IN] index The index in the array.
	 * @return A constant reference to the object.
	 */
	SparseCell& at(std::size_t index);

	/**
	 * Get a reference to the cell at the given index.
	 * This method checks if the index is in the range
	 * and throws an exception if it is not.
	 * @param[IN] index The index in the array.
	 * @return A constant reference to the object.
	 * @throw A std::out_of_range exception if the
	 * given index is larger than the size of the array.
	 */
	const SparseCell& at(std::size_t index) const throw(std::out_of_range);

	/**
	 * Get the current size of the array.
	 * @return The size of the array.
	 */
	std::size_t getSize();

	std::size_t getSize() const;

	/**
	 * Set the size of the array and re-sizes it accordingly.
	 * All newly allocated cells are initialized
	 * to 0.
	 * @param[IN] n The new size of the array.
	 */
	void setSize(std::size_t n);

private:

	/**
	 * Copies the array.
	 * @param[IN/OUT] rhs the other object.
	 */
	void copy(const SparseArray& rhs);

	/**
	 * The current size of the DynArray.
	 */
	std::size_t size_;

	/**
	 * The internal array.
	 */
	SparseCell *array_;

}; /* class Cells */

/**
 * Prints the contents of the Cells into a wostream.
 * @param[IN] out The open stream.
 * @param[IN/OUT] c The Cells array.
 */
std::wostream& operator<<(std::wostream& out, const SparseArray& c);

#endif /* SPARSE_ARRAY_HPP */
