#ifndef SPARSE_ARRAY_MANAGER_HPP
#define SPARSE_ARRAY_MANAGER_HPP

/**
 * @class Header of SparseArrayManager
 * @brief implementing the Sparse Array
 * @author Max Hadersbeck, based on Ideas from Florian Fink
 * *
 * defines the structure and the Header for methods, which are necessary to
 * fill, extend and erase Elements in a Sparse Array
 *
 */

#include <iostream>

/**
 * use Global Variable Debug
 * set Debug greater than one produces Debug output:
 * The bigger value you select, the more output you get
 * Debug=0 means no debug Information
 */
extern int Debug;

#include "sparse_array.hpp"
#include "cellfsa.hpp"
#include "fsa.hpp"

/**
 * This class manages an array of cells and stores TmpStates in it.
  */
class SparseArrayManager {
public:

	using CellFSAPtr = std::shared_ptr<CellFSA>;
	using TransMap_t = std::map< wchar_t, CellFSAPtr>;

	/**
	 * The windows size of the inserting algorithm.
	 * The last free cell is never further away from the
	 * index of the last inserted state than this value.
	 * So (last - free) < WINDOWS_SIZE.
	 * @see cis::SparseTable::last_
	 * @see cis::SparseTable::free_
	 */
	static const std::size_t WINDOW_SIZE = 1000;

	/**
	 * Follow one transition in the table.
	 * @param s The index of the state.
	 * @param c The transition.
	 * @return The target state or 0 if there is no such transition.
	 */

	int delta(int s, wchar_t c) const;
	/**
	 * Constructor.
	 */
	SparseArrayManager();

	/**
	 * Follow one transition in the table.
	 * @param s The index of the state.
	 * @param c The transition.
	 * @return The target state or 0 if there is no such transition.
	 */
	int initialState();

	/**
	 * Initialize the construction of the SparseTable.
	 * This function must be called before inserting any
	 * TmpState to the table.
	 */
	void initConstruction();

	/**
	 * Store a TmpState in the table and update last free index.
	 * Make sure that you have already found a valid position in the table.
	 *  @param s The Cell to store.
	 * @param pos The position where to store the Cell.
	 */
	void storeState(const  CellFSAPtr s, size_t &pos);

	/**
	 * Search for a valid free position in the table to store
	 * a Cell in the Sparse Table.
	 * @param s the Cell to store.
	 * @return The next free position.
	 */
	int findSlot(const CellFSA & s);

	/**
	 * Finish the construction of the SparseTable.
	 * This function must be called when the construction of the table is done.
	 * No more TmpStates can be stored afterward.
	 */
	void finishConstruction();

	/**
	 * Check if a state in the table is final.
	 * @param s The position of the State in the table.
	 * @return True if the state is final or false
	 * if either the state is not final or the given position
	 * does not point to a valid state in the table.
	 */
	bool isFinal(int s);
	bool isFinal(int s) const;

	/**
	 * Get the initial state of the table.
	 * @return The initial state of the table.
	 */
	int initialState() const;

	/**
	 * Get the label of a transition in the table.
	 * @param s The position of the transition in the table.
	 * @return The label of the transition or 0 if the given position
	 * is not a valid transition in the table.
	 */
	wchar_t getLabel(int s);

	/**
	 * Get the offset of the cell at the given position.
	 * @param s The position of the transition or state in the table.
	 * @return The offset of the cell.
	 * @see cis::Cell::offset
	 */
	int  getOffset(int s);

	/**
	 * Get the size of the array.
	 * @return The size of the array.
	 */
	std::size_t getSize();

	/**
	 * Get the size of the Hashtable
	 */
	void setSizeIndexTable(int n);

	/**
	 * Get the number of non empty cells in the SparseTable.
	 * @return The number of non empty cells.
	 */
	std::size_t getFilled();

	/**
	 * Get the number of empty cells in the SparseTable.
	 * @return The number of empty cells.
	 */
	std::size_t getEmpty();

	/**
	 * Check if a token is recognized by the MDA.
	 * @param key The token.
	 * @return true if the token is regognized by the
	 * the MDA; false otherwise.
	 */
	bool contains(const std::wstring& key)const;

private:
	SparseArray table_; ///< data array.
	int *states_index_table_; ///< zum schnellen Lookup, unter welchem Index ist state gespeichert.
	int free_; ///< position of the first free cell.
	int init_; ///< initial state of the automaton.
	int last_; ///< index of the last inserted state.

}; /* class SparseTable */



#endif /* SPARSE_TABLE_HPP */
